package sun.awt.im;

import java.security.PrivilegedAction;
import java.util.prefs.BackingStoreException;
import java.util.Locale;
import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.util.Iterator;
import java.util.ServiceLoader;
import java.security.PrivilegedExceptionAction;
import java.awt.Dialog;
import java.awt.Frame;
import sun.awt.AppContext;
import sun.awt.SunToolkit;
import java.awt.event.InvocationEvent;
import java.lang.reflect.InvocationTargetException;
import java.awt.EventQueue;
import java.awt.im.spi.InputMethodDescriptor;
import java.awt.AWTException;
import sun.awt.InputMethodSupport;
import java.awt.Toolkit;
import java.util.prefs.Preferences;
import java.util.Hashtable;
import java.awt.Component;
import java.util.Vector;

class ExecutableInputMethodManager extends InputMethodManager implements Runnable
{
    private InputContext currentInputContext;
    private String triggerMenuString;
    private InputMethodPopupMenu selectionMenu;
    private static String selectInputMethodMenuTitle;
    private InputMethodLocator hostAdapterLocator;
    private int javaInputMethodCount;
    private Vector<InputMethodLocator> javaInputMethodLocatorList;
    private Component requestComponent;
    private InputContext requestInputContext;
    private static final String preferredIMNode = "/sun/awt/im/preferredInputMethod";
    private static final String descriptorKey = "descriptor";
    private Hashtable<String, InputMethodLocator> preferredLocatorCache;
    private Preferences userRoot;
    
    ExecutableInputMethodManager() {
        this.preferredLocatorCache = new Hashtable();
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        try {
            if ((defaultToolkit instanceof InputMethodSupport)) {
                InputMethodDescriptor inputMethodAdapterDescriptor = ((InputMethodSupport)defaultToolkit).getInputMethodAdapterDescriptor();
                if (inputMethodAdapterDescriptor != null) {
                    this.hostAdapterLocator = new InputMethodLocator(inputMethodAdapterDescriptor, null, null);
                }
            }
        }
        catch (AWTException ex) {}
        this.javaInputMethodLocatorList = new Vector();
        this.initializeInputMethodLocatorList();
    }
    
    synchronized void initialize() {
        ExecutableInputMethodManager.selectInputMethodMenuTitle = Toolkit.getProperty("AWT.InputMethodSelectionMenu", "Select Input Method");
        this.triggerMenuString = ExecutableInputMethodManager.selectInputMethodMenuTitle;
    }
    
    @Override
    public void run() {
        while (!this.hasMultipleInputMethods()) {
            try {
                synchronized (this) {
                    this.wait();
                }
            }
            catch (InterruptedException ex) {}
        }
        while (true) {
            this.waitForChangeRequest();
            this.initializeInputMethodLocatorList();
            try {
                if (this.requestComponent != null) {
                    this.showInputMethodMenuOnRequesterEDT(this.requestComponent);
                }
                else {
                    EventQueue.invokeAndWait(new Runnable() {
                        @Override
                        public void run() {
                            ExecutableInputMethodManager.access$000(ExecutableInputMethodManager.this);
                        }
                    });
                }
            }
            catch (InterruptedException ex2) {}
            catch (InvocationTargetException ex3) {}
        }
    }
    
    private void showInputMethodMenuOnRequesterEDT(Component component) throws InterruptedException, InvocationTargetException {
        if (component == null) {
            return;
        }
        class AWTInvocationLock
        {
        }
        ExecutableInputMethodManager.1AWTInvocationLock notifier = new AWTInvocationLock();
        InvocationEvent event = new InvocationEvent(component, new Runnable() {
            @Override
            public void run() {
                ExecutableInputMethodManager.access$000(ExecutableInputMethodManager.this);
            }
        }, notifier, true);
        AppContext targetToAppContext = SunToolkit.targetToAppContext(component);
        synchronized (notifier) {
            SunToolkit.postEvent(targetToAppContext, event);
            while (!event.isDispatched()) {
                notifier.wait();
            }
        }
        Throwable throwable = event.getThrowable();
        if (throwable != null) {
            throw new InvocationTargetException(throwable);
        }
        return;
    }
    
    @Override
    void setInputContext(InputContext currentInputContext) {
        if (this.currentInputContext == null) {}
        this.currentInputContext = currentInputContext;
    }
    
    @Override
    public synchronized void notifyChangeRequest(Component requestComponent) {
        if (!(requestComponent instanceof Frame) && !(requestComponent instanceof Dialog)) {
            return;
        }
        if (this.requestComponent != null) {
            return;
        }
        this.requestComponent = requestComponent;
        this.notify();
        return;
    }
    
    @Override
    public synchronized void notifyChangeRequestByHotKey(Component parent) {
        while (!(parent instanceof Frame) && !(parent instanceof Dialog)) {
            if (parent == null) {
                return;
            }
            parent = parent.getParent();
        }
        this.notifyChangeRequest(parent);
    }
    
    @Override
    public String getTriggerMenuString() {
        return this.triggerMenuString;
    }
    
    @Override
    boolean hasMultipleInputMethods() {
        return (this.hostAdapterLocator != null && this.javaInputMethodCount > 0) || this.javaInputMethodCount > 1;
    }
    
    private synchronized void waitForChangeRequest() {
        try {
            while (this.requestComponent == null) {
                this.wait();
            }
        }
        catch (InterruptedException ex) {}
    }
    
    private void initializeInputMethodLocatorList() {
        synchronized (this.javaInputMethodLocatorList) {
            this.javaInputMethodLocatorList.clear();
            try {
                AccessController.doPrivileged(new PrivilegedExceptionAction<Object>() {
                    @Override
                    public Object run() {
                        for (InputMethodDescriptor inputMethodDescriptor : ServiceLoader.loadInstalled(InputMethodDescriptor.class)) {
                            ExecutableInputMethodManager.this.javaInputMethodLocatorList.add(new InputMethodLocator(inputMethodDescriptor, inputMethodDescriptor.getClass().getClassLoader(), null));
                        }
                        return null;
                    }
                });
            }
            catch (PrivilegedActionException ex) {
                ex.printStackTrace();
            }
            this.javaInputMethodCount = this.javaInputMethodLocatorList.size();
        }
        if (this.hasMultipleInputMethods()) {
            if (this.userRoot == null) {
                this.userRoot = this.getUserRoot();
            }
        }
        else {
            this.triggerMenuString = null;
        }
    }
    
    private void showInputMethodMenu() {
        if (!this.hasMultipleInputMethods()) {
            this.requestComponent = null;
            return;
        }
        (this.selectionMenu = InputMethodPopupMenu.getInstance(this.requestComponent, ExecutableInputMethodManager.selectInputMethodMenuTitle)).removeAll();
        String currentSelection = this.getCurrentSelection();
        if (this.hostAdapterLocator != null) {
            this.selectionMenu.addOneInputMethodToMenu(this.hostAdapterLocator, currentSelection);
            this.selectionMenu.addSeparator();
        }
        for (int i = 0; i < this.javaInputMethodLocatorList.size(); ++i) {
            this.selectionMenu.addOneInputMethodToMenu((InputMethodLocator)this.javaInputMethodLocatorList.get(i), currentSelection);
        }
        synchronized (this) {
            this.selectionMenu.addToComponent(this.requestComponent);
            this.requestInputContext = this.currentInputContext;
            this.selectionMenu.show(this.requestComponent, 60, 80);
            this.requestComponent = null;
        }
        return;
    }
    
    private String getCurrentSelection() {
        InputContext currentInputContext = this.currentInputContext;
        if (currentInputContext != null) {
            InputMethodLocator inputMethodLocator = currentInputContext.getInputMethodLocator();
            if (inputMethodLocator != null) {
                return inputMethodLocator.getActionCommandString();
            }
        }
        return null;
    }
    
    synchronized void changeInputMethod(String substring) {
        InputMethodLocator inputMethodLocator = null;
        String language = null;
        int index = substring.indexOf(10);
        if (index != -1) {
            language = substring.substring(index + 1);
            substring = substring.substring(0, index);
        }
        if (this.hostAdapterLocator.getActionCommandString().equals(substring)) {
            inputMethodLocator = this.hostAdapterLocator;
        }
        else {
            int i = 0;
            while (i < this.javaInputMethodLocatorList.size()) {
                InputMethodLocator inputMethodLocator2 = (InputMethodLocator)this.javaInputMethodLocatorList.get(i);
                if (inputMethodLocator2.getActionCommandString().equals(substring)) {
                    inputMethodLocator = inputMethodLocator2;
                    break;
                }
                ++i;
            }
        }
        if (inputMethodLocator != null && language != null) {
            String country = "";
            String substring2 = "";
            int index2 = language.indexOf(95);
            if (index2 == -1) {
            }
            else {
                language = language.substring(0, index2);
                int beginIndex = index2 + 1;
                int index3 = language.indexOf(95, beginIndex);
                if (index3 == -1) {
                    country = language.substring(beginIndex);
                }
                else {
                    country = language.substring(beginIndex, index3);
                    substring2 = language.substring(index3 + 1);
                }
            }
            inputMethodLocator = inputMethodLocator.deriveLocator(new Locale(language, country, substring2));
        }
        if (inputMethodLocator == null) {
            return;
        }
        if (this.requestInputContext != null) {
            this.requestInputContext.changeInputMethod(inputMethodLocator);
            this.requestInputContext = null;
            this.putPreferredInputMethod(inputMethodLocator);
        }
        return;
    }
    
    @Override
    InputMethodLocator findInputMethod(Locale locale) {
        InputMethodLocator preferredInputMethod = this.getPreferredInputMethod(locale);
        if (preferredInputMethod != null) {
            return preferredInputMethod;
        }
        if (this.hostAdapterLocator != null && this.hostAdapterLocator.isLocaleAvailable(locale)) {
            return this.hostAdapterLocator.deriveLocator(locale);
        }
        this.initializeInputMethodLocatorList();
        int i = 0;
        while (i < this.javaInputMethodLocatorList.size()) {
            InputMethodLocator inputMethodLocator = (InputMethodLocator)this.javaInputMethodLocatorList.get(i);
            if (inputMethodLocator.isLocaleAvailable(locale)) {
                return inputMethodLocator.deriveLocator(locale);
            }
            ++i;
        }
        return null;
    }
    
    @Override
    Locale getDefaultKeyboardLocale() {
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        if ((defaultToolkit instanceof InputMethodSupport)) {
            return ((InputMethodSupport)defaultToolkit).getDefaultKeyboardLocale();
        }
        return Locale.getDefault();
    }
    
    private synchronized InputMethodLocator getPreferredInputMethod(Locale locale) {
        if (!this.hasMultipleInputMethods()) {
            return null;
        }
        InputMethodLocator inputMethodLocator = (InputMethodLocator)this.preferredLocatorCache.get(locale.toString().intern());
        if (inputMethodLocator != null) {
            return inputMethodLocator;
        }
        String preferredInputMethodNode = this.findPreferredInputMethodNode(locale);
        String preferredInputMethod = this.readPreferredInputMethod(preferredInputMethodNode);
        if (preferredInputMethod != null) {
            if (this.hostAdapterLocator != null && this.hostAdapterLocator.getDescriptor().getClass().getName().equals(preferredInputMethod)) {
                Locale advertisedLocale2 = this.getAdvertisedLocale(this.hostAdapterLocator, locale);
                if (advertisedLocale2 != null) {
                    inputMethodLocator = this.hostAdapterLocator.deriveLocator(advertisedLocale2);
                    this.preferredLocatorCache.put(locale.toString().intern(), inputMethodLocator);
                    return inputMethodLocator;
                }
                return inputMethodLocator;
            }
            else {
                int i = 0;
                while (i < this.javaInputMethodLocatorList.size()) {
                    InputMethodLocator inputMethodLocator2 = (InputMethodLocator)this.javaInputMethodLocatorList.get(i);
                    if (inputMethodLocator2.getDescriptor().getClass().getName().equals(preferredInputMethod)) {
                        Locale advertisedLocale = this.getAdvertisedLocale(inputMethodLocator2, locale);
                        if (advertisedLocale != null) {
                            inputMethodLocator = inputMethodLocator2.deriveLocator(advertisedLocale);
                            this.preferredLocatorCache.put(locale.toString().intern(), inputMethodLocator);
                            return inputMethodLocator;
                        }
                        return inputMethodLocator;
                    }
                    else {
                        ++i;
                    }
                }
                this.writePreferredInputMethod(preferredInputMethodNode, null);
            }
        }
        return null;
    }
    
    private String findPreferredInputMethodNode(Locale locale) {
        if (this.userRoot == null) {
            return null;
        }
        for (String s = "/sun/awt/im/preferredInputMethod/" + this.createLocalePath(locale); !s.equals("/sun/awt/im/preferredInputMethod"); s = s.substring(0, s.lastIndexOf(47))) {
            try {
                if (this.userRoot.nodeExists(s) && this.readPreferredInputMethod(s) != null) {
                    return s;
                }
            }
            catch (BackingStoreException ex) {}
        }
        return null;
    }
    
    private String readPreferredInputMethod(String s) {
        if (this.userRoot == null || s == null) {
            return null;
        }
        return this.userRoot.node(s).get("descriptor", null);
    }
    
    private synchronized void putPreferredInputMethod(InputMethodLocator inputMethodLocator) {
        InputMethodDescriptor descriptor = inputMethodLocator.getDescriptor();
        Object o = inputMethodLocator.getLocale();
        if (o == null) {
            try {
                Locale[] availableLocales = descriptor.getAvailableLocales();
                if (availableLocales.length == 1) {
                    o = availableLocales[0];
                }
                else {
                    return;
                }
            }
            catch (AWTException ex) {
                return;
            }
        }
        if (o.equals(Locale.JAPAN)) {
            o = Locale.JAPANESE;
        }
        if (o.equals(Locale.KOREA)) {
            o = Locale.KOREAN;
        }
        if (o.equals(new Locale("th", "TH"))) {
            o = new Locale("th");
        }
        this.writePreferredInputMethod("/sun/awt/im/preferredInputMethod/" + this.createLocalePath(o), descriptor.getClass().getName());
        this.preferredLocatorCache.put(o.toString().intern(), inputMethodLocator.deriveLocator(o));
    }
    
    private String createLocalePath(Locale locale) {
        String language = locale.getLanguage();
        String country = locale.getCountry();
        String variant = locale.getVariant();
        String s;
        if (!variant.equals("")) {
            s = "_" + language + "/_" + country + "/_" + variant;
        }
        else if (!country.equals("")) {
            s = "_" + language + "/_" + country;
        }
        else {
            s = "_" + language;
        }
        return s;
    }
    
    private void writePreferredInputMethod(String s, String s2) {
        if (this.userRoot != null) {
            Preferences node = this.userRoot.node(s);
            if (s2 != null) {
                node.put("descriptor", s2);
            }
            else {
                node.remove("descriptor");
            }
        }
    }
    
    private Preferences getUserRoot() {
        return (Preferences)AccessController.doPrivileged(new PrivilegedAction<Preferences>() {
            @Override
            public Preferences run() {
                return Preferences.userRoot();
            }
        });
    }
    
    private Locale getAdvertisedLocale(InputMethodLocator inputMethodLocator, Locale locale) {
        Locale locale2 = null;
        if (inputMethodLocator.isLocaleAvailable(locale)) {
            locale2 = locale;
        }
        else if (locale.getLanguage().equals("ja")) {
            if (inputMethodLocator.isLocaleAvailable(Locale.JAPAN)) {
                locale2 = Locale.JAPAN;
            }
            else if (inputMethodLocator.isLocaleAvailable(Locale.JAPANESE)) {
                locale2 = Locale.JAPANESE;
            }
        }
        else if (locale.getLanguage().equals("ko")) {
            if (inputMethodLocator.isLocaleAvailable(Locale.KOREA)) {
                locale2 = Locale.KOREA;
            }
            else if (inputMethodLocator.isLocaleAvailable(Locale.KOREAN)) {
                locale2 = Locale.KOREAN;
            }
        }
        else if (locale.getLanguage().equals("th")) {
            if (inputMethodLocator.isLocaleAvailable(new Locale("th", "TH"))) {
                locale2 = new Locale("th", "TH");
            }
            else if (inputMethodLocator.isLocaleAvailable(new Locale("th"))) {
                locale2 = new Locale("th");
            }
        }
        return locale2;
    }
    
    static void access$000(ExecutableInputMethodManager executableInputMethodManager) {
        executableInputMethodManager.showInputMethodMenu();
    }
}
